/**
 * Author:- mayank
 * Purpose:- This file is used to crate index schema
 */
var async = require('async');
var express = require('express');
var elasticsearch = require('elasticsearch');
var esSchema = require('./esSchema.json');
var _ = require('lodash');
var Deferred = require("promised-io/promise").Deferred;
// var config = require("../utils/config.js")("config/elasticsearch.json")
module.exports = function (app, url, timeOut) {
    var es = {};
    var elasticClient = new elasticsearch.Client({
        host: url,
        log: 'info',
        requestTimeout: timeOut
    });
    es.deleteIndex = function (indexName) {
        app.log.info('Deleting index...');
        return elasticClient.indices.delete({
            index: indexName
        });
    };
    es.deleteRecord = function (indexName, indexType, uuid) {
        app.log.info(indexName, indexType, uuid, 'Deleting index...');
        return elasticClient.delete({
            index: indexName,
            type: indexType,
            id: uuid
        });
    };
    es.updateRecord = function (indexName, indexType, uuid, data) {
        //app.log.info(indexName, indexType, uuid, 'updating record...');
        return elasticClient.update({
            index: indexName,
            type: indexType,
            id: uuid,
            body: data
        });
    };
    es.initIndex = function (indexName) {
        app.log.info('Init indexing...');
        return elasticClient.indices.create({
            index: indexName
        });
    };
    es.indexExists = function (indexName) {
        return elasticClient.indices.exists({
            index: indexName
        });
    };
    es.initMapping = function (indexName, indexType, schema) {
        app.log.info(indexName, 'Mapping...')
        elasticClient.indices.putMapping({
            index: indexName,
            type: indexType,
            body: schema
        });

    };
    es.addIndex = function (data, primeryKey, indexName, indexType) {
        //  app.log.info('addIndex : ' +(route));
        var objbody = {};
        objbody.index = indexName;
        objbody.type = indexType;
        if (primeryKey) objbody.id = primeryKey
        objbody.body = data;
        return elasticClient.index(objbody);

    };
    es.bulkIndex = function (bulkdata) {
        return elasticClient.bulk({
            body: bulkdata
        });
    };
    es.search = function (query, type, indexName) {
        // app.log.info(query.query.bool.must);
        //app.log.info(type);
        app.log.info('Searching...');
        return elasticClient.search({
            index: indexName,
            type: type,
            body: query
        });
    };
    es.searchWithScroll = function (query, type, indexName, scroll) {
        return elasticClient.search({
            index: indexName,
            type: type,
            scroll: scroll,
            body: query
        });
    };
    es.scroll = function (scrollId, scroll) {
        return elasticClient.scroll({
            scrollId: scrollId,
            scroll: scroll
        });
    };
    es.allData = function (query, type, indexName) {
        var dfd = new Deferred();
        query.size = query.size ? query.size : 100;
        var allItems = [];
        es.searchWithScroll(query, type, indexName, '1m')
            .then(function doUntillComplete(results) {
                if (allItems.length == 0) { allItems = results; }
                else {
                    results.hits.hits.forEach(function (hit) {
                        allItems.hits.hits.push(hit);
                    });
                }
                if (results.hits.total > allItems.hits.hits.length) {
                    // ask elasticsearch for the next set of hits from this search
                    es.scroll(
                        results._scroll_id,
                        '1m'
                    ).then(function (results) {
                        doUntillComplete(results);
                    })
                } else {
                    dfd.resolve(allItems);
                    //return allItems;
                }
            }, function (err) {
                app.log.error("err in all api" + err);
                //  return null;
                dfd.reject(err);
            });
        return dfd.promise;
    }
    es.count = function (query, type) {
        app.log.info('Counting...');
        return elasticClient.count({
            index: indexName,
            type: type,
            body: query
        });
    };
    es.getById = function (indexName, indexType, id) {
        var dfd = new Deferred();
        elasticClient.get({
            index: indexName,
            type: indexType,
            id: id,
            ignore: 404
        }, function (err, res) {
            if (err) app.log.error("RB_ES_ERROR IN GETBYID", err)
            dfd.resolve(res);

        });
        return dfd.promise;
    }
    es.documentExists = function (primeryKey, indexName, indexType) {
        app.log.info('Checking document exists...');
        return elasticClient.exists({
            index: indexName,
            type: indexType,
            id: primeryKey
        });
    };
    es.initIndexWithMapping = function (indexName, schema) {
        app.log.info('Init indexing with mapping...' + indexName);
        if (!schema) schema = esSchema[indexName];
        return elasticClient.indices.create({
            index: indexName,
            body: schema
        });
    };
    var bulkData = function (data, indexName, indexType, primeryKey, bulkCount) {
        var bulkdata = [];
        var dfd = new Deferred();
        bulkCount = bulkCount ? bulkCount : 200;
        _.map(data, function name(item) {
            try {
                var header = {};

                header = item.doc ? {
                    update: {
                        _index: indexName,
                        _type: indexType,
                        _id: primeryKey.key ? item.doc ? item.doc[primeryKey.key] : item[primeryKey.key] : undefined
                    }
                } : {
                        index: {
                            _index: indexName,
                            _type: indexType,
                            _id: primeryKey.key ? item.doc ? item.doc[primeryKey.key] : item[primeryKey.key] : undefined
                        }
                    }
                bulkdata.push(header);
                bulkdata.push(item)

            } catch (err) {
                app.log.error(err);
            }
        })
        if (bulkdata.length > 0) {
            var bunchdata = [];
            var j = 1;
            var beesChunks = _.chunk(bulkdata, bulkCount);
            async.eachSeries(beesChunks, function (chunk, cbNext) {
                es.bulkIndex(chunk).then(function (result) {
                    app.log.info('RB_ES: BULK_INDEX- ' + indexName + "/" + indexType, result.items.length,result.errors);
                    if(result.errors){
                        _.map(result.items,function(res){
                            if(res.index){
                                if(res.index.error)app.log.error("RB_ES_ERROR: BULK_INDEX- " + indexName + "/" + indexType,res.index.error.reason)
                            }else if(res.update){
                                if(res.update.error)app.log.error("RB_ES_ERROR: BULK_INDEX- " + indexName + "/" + indexType,res.update.error.reason)
                            }
                        })
                    }
                    cbNext(null, result.items.length);
                }, function (err) {
                    cbNext(err);
                });
            }, function (err, result) {
                if (err) {
                    app.log.error('RB_ES: ERROR_IN_BULK_INDEX' + indexName + "/" + indexType, err);
                    dfd.reject(err)
                }
                else {
                    app.log.info('RB_ES: BULK_INDEX_COMPLATED');
                    dfd.resolve(result);
                }
            });
        }
        return dfd.promise;
    }

    es.formatAndIndex = function (indexName, indexType, data, primeryKey, isInsert, bulkCount, isBulk, onlyUpdate) {
        var dfd = new Deferred();

        var clas = require('./' + indexName);
        var formator = new clas(app);
        if (data.constructor == Array || isBulk != undefined) {
            var formattedData = formator.format(data, null, isInsert);
            if (isInsert == undefined) {
                formattedData = formattedData.map(function (itm) {
                    if (itm.isInsert == false) return document = { "doc": itm }
                    else return itm
                })
            } else {
                if (isInsert == false) {
                    formattedData = formattedData.map(function (itm) {
                        return document = { "doc": itm }
                    })
                }
            }
            bulkData(formattedData, indexName, indexType, primeryKey, bulkCount).then(function (res) {
                dfd.resolve(res);
            }, function (err) {
                dfd.resolve()
            })
        } else {
            if (isInsert) {
                var formattedData = formator.format(data);
                if (formattedData.length > 0) {
                    es.addIndex(formattedData[0], primeryKey.key ? formattedData[0][primeryKey.key] : undefined, indexName, indexType)
                        .then(function (resp) {
                            app.log.info("RB_ES: RECORD_IS_UPDATED_FOR " + indexName + " " + resp._id)
                            dfd.resolve()
                        }, function (err) {
                            app.log.error("RB_ES_ERROR", indexName, err);
                            dfd.resolve()
                        })
                } else {
                    dfd.resolve();
                }
            } else {
                var prmKey = data.attributes ? data.get(primeryKey.key) : data[primeryKey.key]
                var formattedData = formator.format(data, null, isInsert);
                if (formattedData.length > 0) {
                    var document = {
                        "doc": formattedData[0]
                    }
                    es.updateRecord(indexName, indexType, primeryKey.key ? formattedData[0][primeryKey.key] : undefined, document)
                        .then(function (resp) {
                            app.log.info("RB_ES: RECORD_IS_UPDATED_FOR " + indexName + " " + resp._id)
                            dfd.resolve()
                        }, function (err) {
                            if (err.body && err.body.error && err.body.error.type == 'document_missing_exception') {
                                app.log.info("RB_ES_DOCUMENT_NOT_FOUND id " + prmKey + " in " + indexName + " for update now reindexing it")
                                es.addIndex(formattedData[0], primeryKey.key ? formattedData[0][primeryKey.key] : undefined, indexName, indexType)
                                    .then(function (resp) {
                                        app.log.info("RB_ES: RECORD_IS_UPDATED_FOR " + indexName + " " + resp._id)
                                        dfd.resolve()
                                    }, function (err) {
                                        app.log.error("RB_ES_ERROR", indexName, err);
                                        dfd.resolve()
                                    })
                            } else {
                                app.log.error("RB_ES_ERROR", indexName, err);
                                dfd.resolve();
                            }
                        })
                } else {
                    dfd.resolve();
                }
            }


        }
        return dfd.promise;
    }
    es.getHealth = function () {
        app.log.info('cat indices...');
        return elasticClient.cat.indices({
            v: true
        });
    };

    es.deleteNonIDRecord = function (indexName, indexType, customQuery) {
        app.log.info(indexName, indexType, customQuery, 'Deleting index record(s)...');
        return elasticClient.deleteByQuery({
            index: indexName,
            body: customQuery
        });
    };
    return es;
}

