function LEASE(app) {
    var _ = require('lodash');
    var moment = require('moment');
    var self = this;
    self.format = function (input, fromES, isInsert) {
        if ((input.constructor === Array && input[0].table == "lease") || input.table == "lease") {
            return leaseFormatter(input, fromES, isInsert);
        } else if (input[0].table == "branch") {
            return branchFromatter(input, fromES);
        } else if (input[0].table == "bees") {
            return beeFromatter(input, fromES);
        }
    };

    function leaseFormatter(input, fromES, isInsert) {
        var finalData = [];
        var assets = [];
        if (input.constructor === Array) assets = input;
        else assets.push(input);
        if (!isInsert) {
            let asset = assets[0];
            var doc = partialFomat(asset, asset.reportingThreshold);
            if(!JSON.parse(asset.last_known_meta).lbs){
                   delete doc.lbs_location;
                delete doc.lbs_geo_address;
                 delete doc.lbs_ts;  
            }
            finalData.push(doc);
        } else {
            var reportingThreshold = assets[0].reportingThreshold;
            assets.map(function (asset) {
                try {
                    var bee_state;// = JSON.parse(asset.bee_state);
                    var location = {};
                    var geo_address = "";
                    var ts = null
                    var dev_src = "";
                    var bee_info = JSON.parse(asset.bee_info);
                    var asset_data = JSON.parse(asset.asset_data);
                    var lease_data = JSON.parse(asset.lease_data);
                    var jsonObjects = [bee_info, asset_data, lease_data];

                    var inBuilt = 0;
                    if (bee_info.cf_str_dev_src == "SREI") inBuilt = 1;

                    var doc = partialFomat(asset, reportingThreshold);

                    doc.contract = asset.contract
                    doc.asset = asset.asset
                    doc.activeFlag = asset_data.cf_int_custom_status;
                    doc.DPD = asset.deliquency ? asset.deliquency : 0
                    doc.contract_no = asset.contract_no
                    doc.party_name = asset.party_name
                    doc.description = asset.description
                    doc.inBuilt = inBuilt
                    doc.asset_number = asset.asset_number
                    doc.device_name = asset.device_name ? asset.device_name : ""
                    doc.device_number = asset.device_number ? asset.device_number : ""
                    doc.device_uuid = asset.device_uuid ? asset.device_uuid : ""
                    doc.account_id = asset.account_id
                    doc.bucket = asset.bucket
                    if (!doc.location.lat || doc.location.lat == 'null') doc.location = { "lat": 0, "lon": 0 };
                    if (!doc.lbs_location.lat || doc.lbs_location.lat == 'null') doc.lbs_location = { "lat": 0, "lon": 0 };
                    _.map(jsonObjects, function (obj) {
                        for (var property in obj) {
                            if (obj.hasOwnProperty(property)) {
                                doc[property] = obj[property];
                            }
                        }
                    })

                    var latlngVal = /^-?([0-8]?[0-9]|90)\.[0-9]{1,20},-?((1?[0-7]?|[0-9]?)[0-9]|180)\.[0-9]{1,20}$/;

                    if (!latlngVal.test(location.lat + "," + location.lon)) {
                        location.lat = 0;
                        location.lon = 0;
                    }
                    finalData.push(doc);
                } catch (err) {
                    if (err) app.log.error("RB_ES: ES_FORMATTER_ERROR LEASE_INDEX/LEASE", err, asset);
                }
            })
        }
        return finalData;
    }
    function branchFromatter(input, fromES) {
        var finalData = []
        var branches = []
        if (input.constructor === Array) branches = input;
        else branches.push(input)
        branches.map(function (branch) {
            try {
                var doc = {
                    branch: branch.branch,
                    zone: branch.zone,
                    state: branch.state,
                    id: branch.id
                }
                finalData.push(doc);
            } catch (err) {
                if (err) app.log.error("RB_ES: ES_FORMATTER_ERROR LEASE_INDEX/BRANCH", err, branch);
            }
        });
        return finalData;
    }
    function beeFromatter(input, fromES) {
        var finalData = []
        var bees = []
        if (input.constructor === Array) bees = input;
        else bees.push(input)
        bees.map(function (bee) {
            try {
                var doc = {
                    bee_name: bee.bee_name,
                    imei: bee.imei,
                    uuid: bee.uuid
                }
                finalData.push(doc);
            } catch (err) {
                if (err) app.log.error("RB_ES: ES_FORMATTER_ERROR LEASE_INDEX/BEES", err, branch);
            }
        });
        return finalData;
    }
    function partialFomat(asset, reportingThreshold) {
        var last_known_meta = asset.last_known_meta ? JSON.parse(asset.last_known_meta) : null;
        var tempState = asset.state ? typeof asset.state == "string" ? JSON.parse(asset.state) : asset.state : {};
        var tempMonitorState = asset.monitor_state ? typeof asset.monitor_state == "string" ? JSON.parse(asset.monitor_state) : asset.monitor_state : {};
        var monitorState = {
            last_battery_alert: tempMonitorState ? tempMonitorState.last_battery_alert : 'unknown'
        };
        var state = {};
        if (_.has(tempState, 'created_date')) {
            state.created_date = tempState.created_date;
        }
        if (tempState && tempState.last_msg && tempState.last_msg.msg) {
            var lastMsg = {
                msg: {
                    created_date: tempState.last_msg.msg.created_date
                }
            };
            state.last_msg = lastMsg;
        }
        if (tempState && tempState.nocomm_lbs) {
            var tempLbs = [];
            tempLbs.push(tempState.nocomm_lbs[tempState.nocomm_lbs.length - 1]);
            state.nocomm_lbs = tempLbs;
        }
        var location = {};
        var geo_address = "";
        var lbs_geo_address = "";
        var ts = moment();
        var lbs_ts = null;
        var lbs_location = {};
        var rp = 0;
        if (last_known_meta || last_known_meta != null) {
            location = { "lat": last_known_meta.lat != "" && last_known_meta.lat != "#N/A" ? last_known_meta.lat : 0, "lon": last_known_meta.lng != "" && last_known_meta.lng != "#N/A" ? last_known_meta.lng : 0 };
            geo_address = last_known_meta.geo_address;
            if (last_known_meta.last_known_time && last_known_meta.last_known_time != 'null') {
                ts = last_known_meta.last_known_time;
                var tLenght = ts.toString().length;
                ts = tLenght == 10 ? ts * 1000 : ts
                ts = moment(ts);

            }
            if (last_known_meta.lbs) {
               
                var lbs = last_known_meta.lbs;
                lbs_location = { "lat": lbs.lat != "" && lbs.lat != "#N/A" ? lbs.lat : 0, "lon": lbs.lng != "" && lbs.lng != "#N/A" ? lbs.lng : 0 };
                lbs_geo_address = lbs.geo_address;
                if (lbs.last_known_time) {
                    lbs_ts = lbs.last_known_time;
                    var tLenght = lbs_ts.toString().length;
                    lbs_ts = tLenght == 10 ? lbs_ts * 1000 : lbs_ts
                    lbs_ts = moment(lbs_ts);

                }
            }
        }
        if(!ts.isValid()){
            ts=moment();
        }
        if(ts>lbs_ts){
            rp = moment().diff(ts, 'days') < reportingThreshold ? 1 : 0
        }else{
            rp = moment().diff(lbs_ts, 'days') < reportingThreshold ? 1 : 0
        }
        
        ts = ts.format('YYYYMMDDHHmmSS');
        lbs_ts=lbs_ts?lbs_ts.format('YYYYMMDDHHmmSS'):lbs_ts;
        var doc = {
            state: JSON.stringify(state),
            monitor_state: JSON.stringify(monitorState),
            location: location,
            last_known_address: geo_address ? geo_address : "",
            device_imei: asset.device_imei,
            ts: ts,
            reportingFlag: rp,
            lbs_ts:lbs_ts,
            lbs_location:lbs_location,
            lbs_last_known_address: lbs_geo_address ? lbs_geo_address : "",

        };
        return doc;
    }
}



module.exports = LEASE;