
var _ = require('lodash');
var moment = require('moment-timezone');
function SHIPMENT(app) {

    var self = this;
    self.format = function (input, fromES, onlyUpdate) {
        var finalData = [];
        var shipments = [];
        if (input.constructor === Array) shipments = input;
        else shipments.push(input);
        if (shipments.length > 1) {
            var uniqrecords = _.uniq(shipments, function (ship) {
                return ship.attributes.uuid
            })
        } else {
            var uniqrecords = shipments;
        }
        _.map(uniqrecords, function name(ship) {
            try {
                var shipment = {};
                if (ship.attributes) {
                    shipment = ship.attributes;
                    shipment.data = checkNested(shipment, ['data']);
                    shipment.account = ship.relations.account.attributes;
                    shipment.origin = checkNested(ship.relations, ['origin']) ? checkNested(ship.relations.origin.models[0], ['attributes']) : null;
                    shipment.destination = checkNested(ship.relations, ['destination']) ? checkNested(ship.relations.destination.models[0], ['attributes']) : null;
                    if (ship.relations.bees.models.length > 0) shipment.bees = checkNested(ship.relations, ['bees']) ? checkNested(ship.relations.bees.models[0], ['attributes']) : null;
                    else shipment.bees = ship.relations.bees.models;
                    if (shipment.active == 0) shipment.bees = shipment.data.bee?shipment.data.bee:[];
                    if (shipment.bees && shipment.bees.constructor !== Array) shipment.bees = [shipment.bees];
                } else {
                    shipment = ship;
                }
                if (shipment.bees && shipment.bees.length > 0) {
                    shipment.bees[0].state = typeof shipment.bees[0].state == "string" ? JSON.parse(shipment.bees[0].state) : shipment.bees[0].state;
                    shipment.bees[0].monitor_state = typeof shipment.bees[0].monitor_state == "string" ? JSON.parse(shipment.bees[0].monitor_state) : shipment.bees[0].monitor_state;
                }
                if (typeof shipment.data == "string") {
                    shipment.data = JSON.parse(shipment.data);
                }
                shipment.account_name = checkNested(shipment, ['account', 'name']);
                shipment.desti_location_uuid = checkNested(shipment, ['destination', 'data', 'uuid']);
                shipment.orgi_location_uuid = checkNested(shipment, ['origin', 'data', 'uuid'])
                shipment.desti_location_name = checkNested(shipment, ['destination', 'data', 'name'])
                shipment.orgi_location_name = checkNested(shipment, ['origin', 'data', 'name'])
                shipment.created_date = shipment.created_date ? moment.unix(shipment.created_date).format("YYYYMMDD") : null;


                if (checkNested(shipment, ['destination', 'actual_arrival']) != null) {
                    shipment.actual_arrival = moment.unix(shipment.destination.actual_arrival).format("YYYYMMDD");
                    shipment.actual_arrival_timestamp = moment.unix(shipment.destination.actual_arrival).valueOf();
                }
                if (checkNested(shipment, ['origin', 'actual_departure']) != null) {
                    shipment.actual_departure = moment.unix(shipment.origin.actual_departure).format("YYYYMMDD");
                    shipment.actual_departure_timestamp = moment.unix(shipment.origin.actual_departure).valueOf();
                }
                if (checkNested(shipment, ['destination', 'scheduled_arrival']) != null) {
                    shipment.scheduled_arrival = moment.unix(shipment.destination.scheduled_arrival).format("YYYYMMDD");
                    shipment.scheduled_arrival_timestamp = moment.unix(shipment.destination.scheduled_arrival).valueOf();
                }
                if (checkNested(shipment, ['origin', 'scheduled_departure']) != null) {

                    shipment.scheduled_departure = moment.unix(shipment.origin.scheduled_departure).format("YYYYMMDD");
                    shipment.scheduled_departure_timestamp = moment.unix(shipment.origin.scheduled_departure).valueOf();
                }
                shipment.late = checkNested(shipment, ['destination', 'actual_arrival']) == null ? null : checkNested(shipment, ['destination', 'actual_arrival']) > checkNested(shipment, ['origin', 'actual_departure']) ? 1 : 0;
                shipment.shipment_name = checkNested(shipment, ['data', 'name'])
                shipment.shipment_type = shipment.data != null ? returnShipmentType(shipment) : null;
                shipment.account_uuid = checkNested(shipment, ['account', 'uuid']);;
                shipment.beacon_enabled = checkNested(shipment, ['data', 'if-ble-enabled']) == true;
                shipment.ble_List = shipment.data ? returnBleList(shipment) : null;
                shipment.lock_enabled = checkNested(shipment, ['data', 'if-lock-enabled']);
                shipment.eta = shipment.data != null ? getEtaDate(shipment) : null;
                shipment.bee_name = checkNested(shipment.bees[0], ['bee_name']);
                shipment.distance_traveled = checkNested(shipment, ['data', 'distance_travelled']);
                shipment.battery_days = checkNested(shipment.bees[0], ['state', 'battery_remaining_days']);
                shipment.temp = checkNested(shipment.bees[0], ['state', 'temperature', 'temp']);
                shipment.mkt = checkNested(shipment, ['data', 'mkt']);
                shipment.light_state = checkNested(shipment.bees[0], ['state', 'sensor', 'light_state']);
                shipment.pressure = checkNested(shipment.bees[0], ['state', 'PRS']);
                shipment.humidity = checkNested(shipment.bees[0], ['state', 'HUM']);
                shipment.ambient = checkNested(shipment.bees[0], ['state', 'ambient']);
                shipment.tilt_shock = checkNested(shipment.bees[0], ['state', 'TLT'])
                shipment.altitude = checkNested(shipment.bees[0], ['state', 'ALT'])
                shipment.tamper = checkNested(shipment.bees[0], ['state', 'TAM', 'code'])
                shipment.shock = checkNested(shipment.bees[0], ['state', 'SHK', 'shock_code']);
                shipment.last_known_location = checkNested(shipment.bees[0], ['state', 'geo', 'address']);
                shipment.bee_name = checkNested(shipment.bees[0], ['bee_name']);
                shipment.bee_id = checkNested(shipment.bees[0], ['bee_number']);
                shipment.bee_imei = checkNested(shipment.bees[0], ['imei']);
                shipment.mobile_number = checkNested(shipment.bees[0], ['mobile_number']);
                shipment.in_transits = shipment.origin != null ? inTransits(shipment) : null
                shipment.shipment_status=status(shipment);
                shipment.data = shipment.data ? JSON.stringify(shipment.data) : null;
                shipment.bees = JSON.stringify(shipment.bees);
                shipment.origin = JSON.stringify(shipment.origin);
                shipment.destination = JSON.stringify(shipment.destination);
                shipment.beacon_enabled = shipment.beacon_enabled ? shipment.beacon_enabled == true ? 1 : 0 : null
                shipment.lock_enabled = shipment.lock_enabled ? shipment.lock_enabled == true ? 1 : 0 : null
                deleteIfNull(shipment);
                finalData.push(shipment);
            } catch (err) {
                if (err) app.log.error("RB_ES: ES_FORMATTER_ERROR SHIPMENT_INDEX", err, ship);
            }

        })
        if (finalData.length > 1) {
            finalData = _.uniq(finalData, function (item) {
                return item.id
            });

        }
        return finalData;
    }
}

/// return shipment type if as per custom data 
function returnShipmentType(dataSource) {
    var arr = [];
    var str = null;
    if (dataSource.data) {
        if (dataSource.data.custom_data) {
            _.map(dataSource.data.custom_data, function (itm) {
                if (itm.label == "isOrder") arr.push("Order");
                if (itm.label == "isAuto") arr.push("Auto");
            })
        }
        str = arr.join(', ');
        if (!str) {
            str = "Regular";
        }
    }
    return str;
}

/// return BLE List if exist in data
function returnBleList(dataSource) {
    var bles = _.get(dataSource, 'data.BLE');
    if (bles) {
        return _.map(bles, function (ble) {
            return ble.asset_name || ble.imei;
        }).join(', ');
    }
    return 'N/A';
}

///check if key is exist or not in nested object of exist return value else null
function checkNested(obj, keys, returnValue /*, level1, level2, ... levelN*/) {
    var args = keys //    Array.prototype.slice.call(arguments, 1);
    returnValue = returnValue ? returnValue : null
    if (obj) {
        for (var i = 0; i < args.length; i++) {
            if (!obj || !obj[args[i]]) {
                return returnValue;
            }
            obj = obj[args[i]];
            if (i == args.length - 1)
                return obj;
        }
    } else {
        return returnValue;
    }

}
function getShipmentEta(shipment) {
    if (_.has(shipment, 'data.google_eta')) {
        if (shipment.data.google_eta.length) {
            var etaObject = shipment.data.google_eta[shipment.data.google_eta.length - 1];
            var destinationEta = etaObject.time + etaObject.eta;
            return moment(destinationEta);
        }
    }
    return false;
}
function getEtaDate(shipment) {
    var shipmentEta = getShipmentEta(shipment);
    var destinationGeo = getDestinationGeo(shipment);
    if (shipmentEta) {
        return format_date2(shipmentEta.valueOf() / 1000, destinationGeo);
    }
    return false;
}
function getDestinationGeo(shipment) {
    if (_.has(shipment, 'destination.data.geo')) return shipment.destination.data.geo;
    return null;
}
function format_date2(time, geo) {

    if (time === null) return '';
    if (time === 0) return "N/A";

    var timezoneId;
    if (geo && geo.timezone && geo.timezone.timezoneId)
        timezoneId = geo.timezone.timezoneId;
    else if (geo && geo.timezoneId)
        timezoneId = geo.timezoneId;

    if (timezoneId)
        return moment(time * 1000).tz(timezoneId).format('lll z');
    else
        return moment(time * 1000).format('lll z');
}
//// delete key from object if key is null
function deleteIfNull(obj) {
    for (var prop in obj) {
        if (obj[prop] == null) delete obj[prop];
    }
}
function inTransits(dataSource) {
    if (dataSource.origin && dataSource.origin.actual_departure == 0) return 0;
    var start = dataSource.origin.actual_departure;
    var end = dataSource.destination.actual_arrival || moment().unix();
    var hours = Math.round((end - start) / (60 * 60));
    return hours;
}

function status(data) {
    try {
    var scheduled = data.destination.scheduled_arrival * 1000;
    var actual = data.destination.actual_arrival * 1000;
    if (data.destination.actual_arrival != 0 && data.terminated !== 1) {
        // Its Done.
        var msg = 'complete';
        if (Math.abs(scheduled - actual) > 60 * 60 * 1000) {
            if (actual > scheduled) {
                var by = moment.duration(actual - scheduled).humanize();
                msg = msg + ' (late)';
                if (includeDetail) msg = msg + ' by ' + by;
            }
            else {
                var by = moment.duration(scheduled - actual).humanize();
                msg = msg + ' (early)';
                if (includeDetail) msg = msg + ' by ' + by;
            }
        }
        else {
            msg = msg + ' (on time)';
        }
        return msg;
    }

    if (data.origin.actual_departure == 0) return 'not started';


        if (data.terminated === 1 && _.has(data, 'data.terminated.who') && _.has(data, 'data.terminated.reason')) {
            return 'terminated';
        }


    if (!data.bees[0]) {
        // This is a shipment that has been replayed without an allocated bee,
        // but never arrived at its destination!!!
        return 'needs termination';
    }

    var now = moment();
    var b = moment(scheduled).add(6, 'hours');
    if (now.diff(b, 'hours', true) > 6) {
        return "delayed";
    }

    
        var bee = data.bees[0];
        if (bee.monitor_state.last_geo_alert.indexOf("MOVING") > -1) {
            return "moving";
        } else {
            return "stopped";
        }
    } catch (err) {
        return "unknown";
    }

}
module.exports = SHIPMENT;