function SHIPMENT_TRENDS(app) {
    var _ = require('lodash');
    var moment = require('moment');
    var self = this;

    self.format = function (data) {
        // Sort events in cronological order
        let events = _.sortBy(data.shipment.events, function(o) {
            return o.reported_on;
        });


        var finalArray = [];

        for (let idx = 0; idx < events.length - 1; idx++) {
            let prevEvent = events[idx];
            let nextEvent = events[idx+1];

            let eventJson = {};

            try {

                eventJson.shipmentId = data.shipment.meta.uuid;
                eventJson.shipmentType = data.shipment.meta.shipment_type.toString();
                eventJson.accountId = data.shipment.account.uuid;
                eventJson.shipmentOrigin = {
                    lat: data.shipment.meta.origin.lat,
                    lon: data.shipment.meta.origin.lng
                };
                eventJson.shipmentDestination = {
                    lat: data.shipment.meta.destination.lat,
                    lon: data.shipment.meta.destination.lng
                };
                eventJson.shipmentOriginName = data.shipment.meta.origin.name;
                eventJson.shipmentDestinationName = data.shipment.meta.destination.name;
                eventJson.shipmentStartTime = data.shipment.meta.origin.actual_departure;
                eventJson.shipmentEndTime = data.shipment.meta.destination.actual_arrival;

                eventJson.eventStartTime = prevEvent.reported_on;
                eventJson.eventEndTime = nextEvent.reported_on;
                eventJson.eventDuration = nextEvent.reported_on - prevEvent.reported_on;
                eventJson.eventDuration = parseFloat(moment.duration(eventJson.eventDuration, 's').asHours().toFixed(2));
                eventJson.eventOrigin = {
                    lat: prevEvent.data.at.lat,
                    lon: prevEvent.data.at.lng
                };
                eventJson.eventDestination = {
                    lat: nextEvent.data.at.lat,
                    lon: nextEvent.data.at.lng
                };
                eventJson.distanceTravelled = distance(eventJson.eventOrigin.lat, eventJson.eventOrigin.lon, eventJson.eventDestination.lat, eventJson.eventDestination.lon, 'm');
                eventJson.avgSpeed = parseFloat((eventJson.distanceTravelled / eventJson.eventDuration).toFixed(2));


                // Check for distanceTravelled being NaN 
                if (eventJson.distanceTravelled !== eventJson.distanceTravelled) {
                    eventJson.distanceTravelled = 0;
                }

                // Threshold for stoppage is 25mtrs. Should be coming from rb-of but hardcoded for now.
                if (eventJson.distanceTravelled >= 25) {
                    eventJson.legType = 'transit';
                } else {
                    eventJson.legType = 'stoppage';
                }

                // Convert distance travelled to km
                eventJson.distanceTravelled = eventJson.distanceTravelled / 1000;
                eventJson.distanceTravelled = eventJson.distanceTravelled.toFixed(2);

                // Check if this is the start of leg. We do this by checking of the previous legType was the same as this one.
                if (_.last(finalArray)) {
                    eventJson.isStartOfLeg = _.last(finalArray).legType === eventJson.legType ? false : true;
                } else {
                    eventJson.isStartOfLeg = true;
                }

                eventJson.humanizedTimeOfDay = getGreetingTime(moment(eventJson.eventStartTime * 1000));
                eventJson.customId = eventJson.shipmentId + "-" + idx
            
            } catch (err) {
                if (data && data.shipment && data.shipment.meta) {
                    app.log.error("RB_ES: ES_FORMATTER_ERROR SHIPMENT_TRENDS", data.shipment.meta.uuid);
                }
                app.log.error("RB_ES: ES_FORMATTER_ERROR SHIPMENT_TRENDS", err);
                continue;
            }

            let finalJson = {};
            // finalJson = _.merge(staticJson, eventJson);

            finalArray.push(eventJson);
        }

        return finalArray;
    }



    function distance(lat1, lon1, lat2, lon2, unit) {
        var radlat1 = Math.PI * lat1/180
        var radlat2 = Math.PI * lat2/180
        var theta = lon1-lon2
        var radtheta = Math.PI * theta/180
        var dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
        dist = Math.acos(dist)
        dist = dist * 180/Math.PI
        dist = dist * 60 * 1.1515
        if (unit=="K") { dist = dist * 1.609344 }
        if (unit=="m") { dist = dist * 1609.344 }
        return parseInt(dist);
        
    }

    /**
 * Gets humanized time of the day
 * @param  {Moment} m
 */
function getGreetingTime (m) {
	var g = null; //return g
	
	if(!m || !m.isValid()) { return; } //if we can't find a valid or filled moment, we return.
	
	var split_morning = 6;
	var split_afternoon = 12; //24hr time to split the afternoon
	var split_evening = 17; //24hr time to split the evening
	var split_night = 21;
	var currentHour = parseFloat(m.format("HH"));
	
	if (currentHour >= split_night || currentHour < split_morning) {
		g = 'night';
	} else if (currentHour >= split_morning && currentHour < split_afternoon) {
		g = 'morning';
	} else if (currentHour >= split_afternoon && currentHour < split_evening) {
		g = 'afternoon';
	} else if (currentHour >= split_evening && currentHour < split_night) {
		g = 'evening';
	}

	return g;
}
}


module.exports = SHIPMENT_TRENDS;